﻿/*
 * Photostand. Wallpaper.cs
 * Copyright(c) 2008, Shigeki Kaneko. All right reserved.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.Collections.ObjectModel;
using System.Xml;
using Microsoft.Win32;
using System.Windows.Forms;


namespace photostand
{
    class Wallpaper
    {
        public string pictureFile;      // real picture file to set wallpapaer.
        public string displayNameOfPictureFile;  // not real picture file but display name for users to see.
        public int timePeriod;          // time period of changing wallpapers.
        public int dispositionStyle;      // How to set the wallpaper on the display.
        public DateTime dateLastChanged;    // date on which the last changing pictures.
        public int webLocalRatio;       // ratio to describe how percents of files are get from web. 0: Local only, 100: web only.
        public List<DirectoryInfo> dirList; // Directory list in those contain picture files.
        public List<string> keywordList;    // list of keywords.
        public Webpicture webpicture;   // web if to get pictures from the server.
        public int sequentialMode;      // 0: random, 1: sequential.
        public int folderRoot;          // 0: MyPicture, 1: null = MyComputer.
        public int folderRecursive;    // 1: folder should be searched recursively.
        public int continueAfterDirectChange;    // 0: not continue, 1: continue. continue the sequencial selecting after selecting a picture for quick change, if the picture is in the selected folders.
        public bool configIsChanged;    // 1: Configuration is changed by the user.

        private string configFile;      // Configuration file on appdata dir.
        private string logFile;         // Log file on appdata dir.
        private string tmpPicFile;      // Temporary picture file for images with format needs to be changed.
        private Random random;          // random number generator.
        private bool isLocalPicture;    // Which the current picture is local file or not.
        private int pictureSequenceNumber;   // sequence number of the picture currently displayed.

        public Wallpaper()
        {
            // int pid = Process.GetCurrentProcess().Id;
            int rseed = (int)DateTime.Now.ToBinary();
            // this.random = new Random(rseed | pid);
            this.random = new Random(rseed);
            try {
                string appDir = String.Format(@"{0}\{1}\{2}", Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "Nikep", "photostand");
                if (!Directory.Exists(appDir)) {
                    Directory.CreateDirectory(appDir);
                }
                this.configFile = String.Format(@"{0}\{1}", appDir, "phstdcfg.xml");
                this.logFile = String.Format(@"{0}\{1}", appDir, "phstdlog.xml");
                this.tmpPicFile = String.Format(@"{0}\{1}", appDir, "_phsttmp.bmp");
                this.webpicture = new Webpicture();
            } catch (Exception ex) {
                Debug.WriteLine("Wallpaper: CreateDir" + ex.ToString());
                return;
            }

            // Initialize the variables
            this.dirList = new List<DirectoryInfo>();
            this.keywordList = new List<string>();
            this.pictureFile = this.displayNameOfPictureFile = null;
            this.folderRoot = 0;
            this.sequentialMode = 0;
            this.pictureSequenceNumber = -1;
            this.timePeriod = 3;    // 10 min.
            this.dispositionStyle = 4;  // covering
            this.dateLastChanged = DateTime.Now;
            this.webLocalRatio = 0; // [0-5] local 100%
            this.folderRecursive = 0; // not recursive
            this.continueAfterDirectChange = 0; // not continue
            this.configIsChanged = false;
            this.pictureFile = String.Empty;

            this.LoadConfig();
            this.LoadLog();
        }

        /// <summary>
        /// Change the wallpaper to one which is randomly selected.
        /// </summary>
        /// <returns>int: 0 for no error, otherwise some errors</returns>
        public int RandomSetWallpaper()
        {
            int rdn = this.random.Next(100);
            int rt;

            Debug.WriteLine("RandomSetWallpaper:(" + rdn.ToString() + "/ " + (this.webLocalRatio * 25).ToString() + ")\n");
            if (rdn >= this.webLocalRatio * 25) {
                // Selecting from local file.
                rt = this.RandomSetLocalWallpaper();
            } else {
                // getting from the web.
                rt = this.RandomSetWebWallpaper();
            }
            return rt;
        }

        /// <summary>
        /// Change the wallpaper to one which is randomly selected.
        /// </summary>
        /// <returns>int: 0 for no error, otherwise some errors</returns>
        private int RandomSetLocalWallpaper()
        {
            List<FileInfo> pFiles = new List<FileInfo>();
            foreach (DirectoryInfo di in this.dirList) {
                try {
                    FileInfo[] fis = di.GetFiles();
                    foreach (FileInfo fi in fis) {
                        if (fi.Extension == ".bmp" || fi.Extension == ".jpg" || fi.Extension == ".png") {
                            // Debug.WriteLine("Add " + fi.Name);
                            pFiles.Add(fi);
                        }
                    }
                    if (this.folderRecursive == 1) {
                        DirectoryInfo[] drs = di.GetDirectories("*", System.IO.SearchOption.AllDirectories);
                        foreach (DirectoryInfo dr in drs) {
                            // FileInfo[] fifs = dr.GetFiles();
                            foreach (FileInfo fi in dr.GetFiles()) {
                                if (fi.Extension == ".bmp" || fi.Extension == ".jpg" || fi.Extension == ".png") {
                                    // Debug.WriteLine("Add " + fi.Name);
                                    pFiles.Add(fi);
                                }
                            }
                        }
                    }
                    Debug.WriteLine("pFiles[" + di + "](" + pFiles.Count + ")");
                } catch (Exception ept) {
                    Debug.WriteLine("Error: " + ept.ToString());
                    // MessageBox.Show("Error on Checking directories: " + ept.ToString());
                }
            }

            // random selection from the list of all pictures.
            if (pFiles.Count <= 0) {
                Debug.WriteLine("Error: RandomSetWallpaper: no picture files.");
                return -1;
            }
            if (this.sequentialMode == 0) {
                int rndnbr = this.random.Next(pFiles.Count);
                Debug.WriteLine("Random Number for file: " + rndnbr + " / " + pFiles.Count);
                this.pictureFile = this.displayNameOfPictureFile = pFiles[rndnbr].FullName;
            } else {
                if (this.pictureSequenceNumber == -1 || this.pictureSequenceNumber >= pFiles.Count) {
                    this.pictureSequenceNumber = 0;
                }
                this.pictureFile = this.displayNameOfPictureFile = pFiles[this.pictureSequenceNumber++].FullName;
            }
            this.isLocalPicture = true;
            int rt = ChangeWallpaper();
            return rt;
        }

        /// <summary>
        /// Get and change the web wallpaper to one which is randomly selected.
        /// </summary>
        /// <returns>int: 0 for no error, otherwise some errors</returns>
        private int RandomSetWebWallpaper()
        {
            int rt, n;

            n = this.keywordList.Count;
            // random selection from the list of all pictures.
            if (n <= 0) {
                Debug.WriteLine("Error: RandomSetWebWallpaper: no keyword.");
            } else {
                int rd = this.random.Next(n);
                this.webpicture.keyword = this.keywordList[rd];
            }

            rt = this.webpicture.WebGetPicture();
            if (rt == 0) {
                // successfully getting the picture.
                this.pictureFile = this.webpicture.webPictureFile;
                this.displayNameOfPictureFile = this.webpicture.pictureUri;
                this.isLocalPicture = false;
                rt = this.ChangeWallpaper();
            } else {
                // In case of failing to get picture, set local files. 
                rt = this.RandomSetLocalWallpaper();
            }

            return rt;
        }

        /// <summary>
        /// changing wallpaper to the picture file in the argument.
        /// </summary>
        /// <returns>int: 0 success, otherwise some errors</returns>
        public int ChangeWallpaper()
        {
            Bitmap bm = null;
            if (this.pictureFile == null) {
                Debug.WriteLine("ChangeWallpaper: this.pictureFile is (null)\n");
                return -1;
            }
            string picFile = this.pictureFile;
            string newWallpaperStyle, newTileWallpaper;
            int dspStyle = this.dispositionStyle;

            Debug.WriteLine("ChangeWallpaper(" + this.pictureFile + ") " + this.dispositionStyle);
            try {
                uint ort;   // Orientation of the picture, suggested by the exif.
                // Check dspStyle to know if modifying the picture is necessary or not.
                switch (dspStyle) {
                case 0:
                case 1:
                case 2:
                    // transencoding the pciture file if not Windows BMP file.
                    //if (Path.GetExtension(this.pictureFile).ToLower() != ".bmp") {
                    bm = new Bitmap(this.pictureFile);
                    // Check orientation hint of EXIF to know if rotation of the picture is necessary or not.
                    ort = GetPictureOrientation(bm);
                    switch (ort) {
                    case 2:
                        bm.RotateFlip(RotateFlipType.RotateNoneFlipX);
                        break;
                    case 3:
                        bm.RotateFlip(RotateFlipType.Rotate180FlipNone);
                        break;
                    case 4:
                        bm.RotateFlip(RotateFlipType.Rotate180FlipX);
                        break;
                    case 5:
                        bm.RotateFlip(RotateFlipType.Rotate90FlipX);
                        break;
                    case 6:
                        bm.RotateFlip(RotateFlipType.Rotate90FlipNone);
                        break;
                    case 7:
                        bm.RotateFlip(RotateFlipType.Rotate270FlipX);
                        break;
                    case 8:
                        bm.RotateFlip(RotateFlipType.Rotate270FlipNone);
                        break;
                    default:    // 0 or 1.
                        break;
                    }
                    picFile = this.tmpPicFile;
                    Debug.WriteLine("pictureFile = " + picFile);
                    bm.Save(picFile, System.Drawing.Imaging.ImageFormat.Bmp);
                    //}
                    break;
                case 3:
                case 4:
                default:
                    bm = new Bitmap(this.pictureFile);
                    // Check orientation hint of EXIF to know if rotation of the picture is necessary or not.
                    ort = GetPictureOrientation(bm);
                    switch (ort) {
                    case 2:
                        bm.RotateFlip(RotateFlipType.RotateNoneFlipX);
                        break;
                    case 3:
                        bm.RotateFlip(RotateFlipType.Rotate180FlipNone);
                        break;
                    case 4:
                        bm.RotateFlip(RotateFlipType.Rotate180FlipX);
                        break;
                    case 5:
                        bm.RotateFlip(RotateFlipType.Rotate90FlipX);
                        break;
                    case 6:
                        bm.RotateFlip(RotateFlipType.Rotate90FlipNone);
                        break;
                    case 7:
                        bm.RotateFlip(RotateFlipType.Rotate270FlipX);
                        break;
                    case 8:
                        bm.RotateFlip(RotateFlipType.Rotate270FlipNone);
                        break;
                    default:    // 0 or 1.
                        break;
                    } 
                    Rectangle dp = Screen.PrimaryScreen.Bounds;
                    int wt, ht;
                    int widthForDpyHeight = bm.Width * dp.Height / bm.Height;
                    Debug.WriteLine("widthForDpyHeight = " + widthForDpyHeight + " ScreenSize (" + dp.Width + "," + dp.Height + ")" + " PictureSize (" + bm.Width + "," + bm.Height + ")");
                    if (widthForDpyHeight > dp.Width) {
                        if (this.dispositionStyle == 3) {
                            wt = dp.Width;
                            ht = bm.Height * dp.Width / bm.Width;
                        } else {
                            wt = bm.Width * dp.Height / bm.Height;
                            ht = dp.Height;
                        }
                    } else if (widthForDpyHeight < dp.Width) {
                        if (this.dispositionStyle == 3) {
                            wt = bm.Width * dp.Height / bm.Height;
                            ht = dp.Height;
                        } else {
                            Bitmap bmpcrop = bm;
                            // Portrait such as human portrait always has the center in the upper half.
                            if (!this.isLocalPicture && (bm.Width * dp.Height / (bm.Height * 75 / 100)) < dp.Width) {
                                Rectangle cropArea = new Rectangle(0, 0, bm.Width, bm.Height * 75 / 100);
                                bmpcrop = bm.Clone(cropArea, bm.PixelFormat);
                                bm.Dispose();
                                bm = bmpcrop;
                            }
                            wt = dp.Width;
                            ht = bmpcrop.Height * dp.Width / bmpcrop.Width;
                        }
                    } else {
                        // image size is completely the same as the display size.
                        wt = -1;
                        ht = -1;
                        dspStyle = 0;
                    }
                    Debug.WriteLine("ScreenSize (" + dp.Width + "," + dp.Height + ")" + " PictureSize (" + bm.Width + "," + bm.Height + ")" + " Resize to (" + wt + "," + ht + ")");
                    if (wt > 0) {
                        Bitmap nbm = new Bitmap(bm, wt, ht);
                        picFile = this.tmpPicFile;
                        Debug.WriteLine("pictureFile = " + picFile + " Resize to (" + wt + "," + ht + ")");
                        nbm.Save(picFile, System.Drawing.Imaging.ImageFormat.Bmp);
                        nbm.Dispose();
                    } else if (Path.GetExtension(this.pictureFile).ToLower() != ".bmp") {
                        picFile = this.tmpPicFile;
                        Debug.WriteLine("pictureFile = " + picFile);
                        bm.Save(picFile, System.Drawing.Imaging.ImageFormat.Bmp);
                    }
                    break;
                }

                // setting wallpaper style
                RegistryKey rk = Registry.CurrentUser.OpenSubKey(@"Control Panel\Desktop", true);
                string curWallpaperStyle, curTileWallpaper;
                switch (dspStyle) {
                case 0:   // stretched
                    newWallpaperStyle = "2";
                    newTileWallpaper = "0";
                    break;
                case 1:   // centered
                case 3:
                case 4:
                    newWallpaperStyle = "1";
                    newTileWallpaper = "0";
                    break;
                case 2:   // tiled
                    newWallpaperStyle = "1";
                    newTileWallpaper = "1";
                    break;
                default:
                    newWallpaperStyle = "2";
                    newTileWallpaper = "0";
                    break;
                }
                curWallpaperStyle = (string)rk.GetValue(@"WallpaperStyle");
                curTileWallpaper = (string)rk.GetValue(@"TileWallpaper");
                if (!newWallpaperStyle.Equals(curWallpaperStyle)) {
                    rk.SetValue(@"WallpaperStyle", newWallpaperStyle);
                }
                if (!newTileWallpaper.Equals(curTileWallpaper)) {
                    rk.SetValue(@"TileWallpaper", newTileWallpaper);
                }

                // setting the picture to the wallpaper.
                int rtn = WinAPI.SystemParametersInfo(WinAPI.SPI_SETDESKWALLPAPER, 1, picFile, WinAPI.SPIF_SENDCHANGE);
                if (rtn == 0) {
                    Debug.WriteLine("SystemParametersInfo: " + picFile + " = " + rtn.ToString());
                    throw new Exception("Error in SystemParametersInfo" + picFile.ToString());
                } else {
                    this.dateLastChanged = DateTime.Now;
                    this.SaveLog();
                }
            } catch (Exception ept) {
                Debug.WriteLine("Error: " + ept.ToString());
                // MessageBox.Show("Can't change wallpapers: " + ept.ToString());
                return 1;
            } finally {
                if (bm != null) {
                    bm.Dispose();
                }
            }
            return 0;
        }

        /// <summary>
        /// Save photostand configuration data in the config file.
        /// </summary>
        public int ChangeDirectWallpaper(string pictureFile)
        {
            this.pictureFile = this.displayNameOfPictureFile = pictureFile;

            // change the wallpaper
            int rt = this.ChangeWallpaper();

            // If the picture is in the picture folders the next picture should be the next of it.
            if (rt >= 0 && this.sequentialMode == 1 && this.continueAfterDirectChange == 1) {
                int cnt = 0;
                bool isIncluded = false;
                List<FileInfo> pFiles = new List<FileInfo>();
                foreach (DirectoryInfo di in this.dirList) {
                    try {
                        FileInfo[] fis = di.GetFiles();
                        foreach (FileInfo fi in fis) {
                            if (fi.Extension == ".bmp" || fi.Extension == ".jpg" || fi.Extension == ".png") {
                                cnt++;
                                if (fi.FullName.CompareTo(pictureFile) == 0) {
                                    Debug.WriteLine("File found " + pictureFile + ": " + fi.Name);
                                    isIncluded = true;
                                    goto file_found_label;
                                }
                            }
                        }
                        if (this.folderRecursive == 1) {
                            DirectoryInfo[] drs = di.GetDirectories("*", System.IO.SearchOption.AllDirectories);
                            foreach (DirectoryInfo dr in drs) {
                                // FileInfo[] fifs = dr.GetFiles();
                                foreach (FileInfo fi in dr.GetFiles()) {
                                    if (fi.Extension == ".bmp" || fi.Extension == ".jpg" || fi.Extension == ".png") {
                                        cnt++;
                                        if (fi.FullName.CompareTo(pictureFile) == 0) {
                                            Debug.WriteLine("File found " + pictureFile + ": " + fi.Name);
                                            isIncluded = true;
                                            goto file_found_label;
                                        }
                                    }
                                }
                            }
                        }

                    } catch (Exception ept) {
                        Debug.WriteLine("Error: " + ept.ToString());
                        // MessageBox.Show("Error on Checking directories: " + ept.ToString());
                    }
                }

            file_found_label:
                if (isIncluded) {
                    Debug.WriteLine("pictureSequenceNumber: " + cnt.ToString());
                    this.pictureSequenceNumber = cnt;
                }
            }
            return 0;
        }

        /// <summary>
        /// Save photostand configuration data in the config file.
        /// </summary>
        public void SaveConfig()
        {
            XmlWriterSettings xs = new XmlWriterSettings();
            xs.Encoding = System.Text.Encoding.UTF8;
            xs.Indent = true;
            // xs.IndentChars = ("\t");
            try {
                XmlWriter xw = System.Xml.XmlWriter.Create(this.configFile, xs);

                xw.WriteStartElement("photostandconfig");
                xw.WriteStartElement("changeperiod");
                xw.WriteValue(this.timePeriod);
                xw.WriteEndElement();

                xw.WriteStartElement("dispositionstyle");
                xw.WriteValue(this.dispositionStyle);
                xw.WriteEndElement();

                xw.WriteStartElement("weblocalratio");
                xw.WriteValue(this.webLocalRatio);
                xw.WriteEndElement();

                xw.WriteStartElement("sequentialmode");
                xw.WriteValue(this.sequentialMode);
                xw.WriteEndElement();

                xw.WriteStartElement("continueAfterDirectChange");
                xw.WriteValue(this.continueAfterDirectChange);
                xw.WriteEndElement();

                xw.WriteStartElement("folderrecursive");
                xw.WriteValue(this.folderRecursive);
                xw.WriteEndElement();

                xw.WriteStartElement("folderroot");
                xw.WriteValue(this.folderRoot);
                xw.WriteEndElement();

                xw.WriteStartElement("directorylist");
                foreach (DirectoryInfo di in this.dirList) {
                    xw.WriteStartElement("directory");
                    xw.WriteValue(di.ToString());
                    xw.WriteEndElement();
                }
                xw.WriteEndElement();

                xw.WriteStartElement("keywordlist");
                foreach (string s in this.keywordList) {
                    xw.WriteStartElement("keyword");
                    xw.WriteValue(s);
                    xw.WriteEndElement();
                }
                xw.WriteEndElement();

                xw.WriteEndElement();
                xw.Close();
            } catch (Exception ex) {
                Debug.WriteLine("SaveConfig: " + ex.ToString());
            }
            // clear the configIsChanged flag to show the save isn't necessary anymore.
            this.configIsChanged = false;
        }

        /// <summary>
        /// Load photostand configuration data from the config file.
        /// </summary>
        public void LoadConfig()
        {
            try {
                XmlReader xr = XmlReader.Create(this.configFile);
                string s;
                while (xr.Read()) {
                    if (xr.NodeType == XmlNodeType.Element) {
                        switch (xr.LocalName) {
                            case "changeperiod":
                                this.timePeriod = int.Parse(xr.ReadString());
                                break;
                            case "dispositionstyle":
                                this.dispositionStyle = int.Parse(xr.ReadString());
                                break;
                            case "weblocalratio":
                                this.webLocalRatio = int.Parse(xr.ReadString());
                                break;
                            case "sequentialmode":
                                this.sequentialMode = int.Parse(xr.ReadString());
                                break;
                            case "continueAfterDirectChange":
                                this.continueAfterDirectChange = int.Parse(xr.ReadString());
                                break;
                            case "folderrecursive":
                                this.folderRecursive = int.Parse(xr.ReadString());
                                break;
                            case "folderroot":
                                this.folderRoot = int.Parse(xr.ReadString());
                                break;
                            case "directorylist":
                                break;
                            case "directory":
                                s = xr.ReadString();
                                DirectoryInfo di = new DirectoryInfo(s);
                                if (!di.Exists) {
                                    Debug.WriteLine("Main: DIR(" + s + ") is specified, but not exist.");
                                    continue;
                                }
                                this.dirList.Add(di);
                                break;
                            case "keywordlist":
                                break;
                            case "keyword":
                                s = xr.ReadString();
                                this.keywordList.Add(s);
                                break;
                            default:
                                break;
                        }
                    }
                }
                xr.Close();
                //-- for DEBUG
                // webpicture.keyword = "nature";
            } catch (Exception ex) {
                Debug.WriteLine("LoadConfig: " + ex.ToString());
                List<DirectoryInfo> dis = new List<DirectoryInfo>();
                // dis.Add(new DirectoryInfo(@"C:\Users\Public\Pictures\Sample Pictures"));
                if ((Environment.OSVersion.Platform == PlatformID.Win32NT) && ((Environment.OSVersion.Version.Major > 5) || ((Environment.OSVersion.Version.Major == 5) && (Environment.OSVersion.Version.Minor >= 1)))) {
                    dis.Add(new DirectoryInfo(@"C:\Users\Public\Pictures\Sample Pictures"));
                } else {
                    dis.Add(new DirectoryInfo(@"C:\Documents and Settings\All Users\Documents\My Pictures\Sample Pictures"));
                }
                // for .NET 4.x dis.Add(new DirectoryInfo(Environment.GetFolderPath(Environment.SpecialFolder.CommonPictures) + @"\Sample Pictures"));
                dis.Add(new DirectoryInfo(Environment.GetFolderPath(Environment.SpecialFolder.MyPictures)));
                foreach (DirectoryInfo di in dis) {
                    if (di.Exists) {
                        this.dirList.Add(di);
                    }
                }
            }
        }

        
        /// <summary>
        /// Reset the program setting and going back to the default configuration.
        /// </summary>
        public void ResetConfig()
        {
            // Remove the log file and config file, after the function call, it will restart.
            File.Delete(this.logFile);
            File.Delete(this.configFile);
        }
 
        /// <summary>
        /// Save date to the log file to check expiration of the time period in the future.
        /// </summary>
        private void SetTimeLastChanged()
        {
            this.dateLastChanged = DateTime.Now;
            this.LoadLog();
        }

        /// <summary>
        /// Save date to the log file to check expiration of the time period in the future.
        /// </summary>
        public void SaveLog()
        {
            XmlWriterSettings xs = new XmlWriterSettings();
            xs.Encoding = System.Text.Encoding.UTF8;
            xs.Indent = true;
            // xs.IndentChars = ("\t");
            try {
                XmlWriter xw = XmlWriter.Create(this.logFile, xs);
                xw.WriteStartElement("photostandlog");
                xw.WriteStartElement("datelastchanged");
                xw.WriteValue(this.dateLastChanged.ToString());
                xw.WriteEndElement();
                if (this.sequentialMode == 1 && this.pictureSequenceNumber != -1) {
                    xw.WriteStartElement("sequencenumber");
                    xw.WriteValue(this.pictureSequenceNumber.ToString());
                    xw.WriteEndElement();
                }
                xw.WriteStartElement("picturefile");
                xw.WriteValue(this.pictureFile.ToString());
                xw.WriteEndElement();

                xw.WriteEndElement();
                xw.Close();
            } catch (Exception ex) {
                Debug.WriteLine("SaveLog: " + ex.ToString());
            }
        }

        /// <summary>
        /// Load date to the log file to check expiration of the time period in the future.
        /// </summary>
        public void LoadLog()
        {
            try {
                XmlReader xr = XmlReader.Create(this.logFile);
                while (xr.Read()) {
                    if (xr.NodeType == XmlNodeType.Element) {
                        if (xr.LocalName == "datelastchanged") {
                            this.dateLastChanged = DateTime.Parse(xr.ReadString());
                        }
                    }
                    if (xr.NodeType == XmlNodeType.Element) {
                        if (xr.LocalName == "sequencenumber") {
                            this.pictureSequenceNumber = int.Parse(xr.ReadString());
                        }
                    }
                    if (xr.NodeType == XmlNodeType.Element) {
                        if (xr.LocalName == "picturefile") {
                            this.pictureFile = xr.ReadString();
                        }
                    }
                }
                xr.Close();
            } catch (Exception ex) {
                Debug.WriteLine("SaveLog: " + ex.ToString());
            }
        }

        /// <summary>
        /// Determine the orientation of the picture by reading exif
        /// </summary>
        private ushort GetPictureOrientation (Bitmap bmp)
        {
            if (bmp == null) {
                return 0;
            }
            foreach (PropertyItem itm in bmp.PropertyItems) {
                if (itm.Id == 0x0112) { /// PropertyItem.PropertyTagOrientation) { // 0x0112
                    Debug.WriteLine("GetPictureOrientation:" + BitConverter.ToUInt16(itm.Value, 0).ToString() + ".");
                    return BitConverter.ToUInt16(itm.Value, 0);
                }
            }
            return 0;
        }
    }

    /// <summary>
    /// WinAPI fnction: special declaration to be called from C# code.
    /// </summary>
    public class WinAPI
    {
        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern int SystemParametersInfo(int uAction, int uParam, string lpvParam, int fuWinIni);
        public const int SPI_SETDESKWALLPAPER = 20;
        public const int SPIF_SENDCHANGE = 0x2;
    }
}
